import json
import multiprocessing as mp
import typing as t
import uuid
from dataclasses import dataclass

from unstructured_ingest.enhanced_dataclass import enhanced_field
from unstructured_ingest.error import DestinationConnectionError, WriteError
from unstructured_ingest.interfaces import (
    AccessConfig,
    BaseConnectorConfig,
    BaseDestinationConnector,
    ConfigSessionHandleMixin,
    IngestDocSessionHandleMixin,
    WriteConfig,
)
from unstructured_ingest.logger import logger
from unstructured_ingest.utils.data_prep import batch_generator, flatten_dict
from unstructured_ingest.utils.dep_check import requires_dependencies

if t.TYPE_CHECKING:
    from qdrant_client import QdrantClient


@dataclass
class QdrantAccessConfig(AccessConfig):
    api_key: t.Optional[str] = enhanced_field(sensitive=True)


@dataclass
class SimpleQdrantConfig(ConfigSessionHandleMixin, BaseConnectorConfig):
    collection_name: str
    location: t.Optional[str] = None
    url: t.Optional[str] = None
    port: t.Optional[int] = 6333
    grpc_port: t.Optional[int] = 6334
    prefer_grpc: t.Optional[bool] = False
    https: t.Optional[bool] = None
    prefix: t.Optional[str] = None
    timeout: t.Optional[float] = None
    host: t.Optional[str] = None
    path: t.Optional[str] = None
    force_disable_check_same_thread: t.Optional[bool] = False
    access_config: t.Optional[QdrantAccessConfig] = None


@dataclass
class QdrantWriteConfig(WriteConfig):
    batch_size: int = 50
    num_processes: int = 1


@dataclass
class QdrantDestinationConnector(IngestDocSessionHandleMixin, BaseDestinationConnector):
    write_config: QdrantWriteConfig
    connector_config: SimpleQdrantConfig
    _client: t.Optional["QdrantClient"] = None

    @property
    def qdrant_client(self):
        if self._client is None:
            self._client = self.create_client()
        return self._client

    def initialize(self):
        ...  # fmt: skip

    @requires_dependencies(["qdrant_client"], extras="qdrant")
    def create_client(self) -> "QdrantClient":
        from qdrant_client import QdrantClient

        client = QdrantClient(
            location=self.connector_config.location,
            url=self.connector_config.url,
            port=self.connector_config.port,
            grpc_port=self.connector_config.grpc_port,
            prefer_grpc=self.connector_config.prefer_grpc,
            https=self.connector_config.https,
            api_key=(
                self.connector_config.access_config.api_key
                if self.connector_config.access_config
                else None
            ),
            prefix=self.connector_config.prefix,
            timeout=self.connector_config.timeout,
            host=self.connector_config.host,
            path=self.connector_config.path,
            force_disable_check_same_thread=self.connector_config.force_disable_check_same_thread,
        )

        return client

    @DestinationConnectionError.wrap
    def check_connection(self):
        self.qdrant_client.get_collections()

    @DestinationConnectionError.wrap
    @requires_dependencies(["qdrant_client"], extras="qdrant")
    def upsert_batch(self, batch: t.List[t.Dict[str, t.Any]]):
        from qdrant_client import models

        client = self.qdrant_client
        try:
            points: list[models.PointStruct] = [models.PointStruct(**item) for item in batch]
            response = client.upsert(
                self.connector_config.collection_name, points=points, wait=True
            )
        except Exception as api_error:
            raise WriteError(f"Qdrant error: {api_error}") from api_error
        logger.debug(f"results: {response}")

    def write_dict(self, *args, elements_dict: t.List[t.Dict[str, t.Any]], **kwargs) -> None:
        logger.info(
            f"Upserting {len(elements_dict)} elements to "
            f"{self.connector_config.collection_name}",
        )

        qdrant_batch_size = self.write_config.batch_size

        logger.info(f"using {self.write_config.num_processes} processes to upload")
        if self.write_config.num_processes == 1:
            for chunk in batch_generator(elements_dict, qdrant_batch_size):
                self.upsert_batch(chunk)

        else:
            with mp.Pool(
                processes=self.write_config.num_processes,
            ) as pool:
                pool.map(self.upsert_batch, list(batch_generator(elements_dict, qdrant_batch_size)))

    def normalize_dict(self, element_dict: dict) -> dict:
        return {
            "id": str(uuid.uuid4()),
            "vector": element_dict.pop("embeddings", {}),
            "payload": {
                "text": element_dict.pop("text", None),
                "element_serialized": json.dumps(element_dict),
                **flatten_dict(
                    element_dict,
                    separator="-",
                    flatten_lists=True,
                ),
            },
        }



import typing as t
import warnings
from dataclasses import dataclass
from urllib.parse import urlparse

from unstructured_ingest.interfaces import BaseSourceConnector
from unstructured_ingest.logger import logger
from unstructured_ingest.runner.base_runner import Runner
from unstructured_ingest.runner.utils import update_download_dir_remote_url

if t.TYPE_CHECKING:
    from unstructured_ingest.connector.fsspec.fsspec import SimpleFsspecConfig


@dataclass
class FsspecRunner(Runner):
    connector_config: "SimpleFsspecConfig"

    def update_read_config(self):
        self.read_config.download_dir = update_download_dir_remote_url(
            connector_name="fsspec",
            read_config=self.read_config,
            remote_url=self.fsspec_config.remote_url,  # type: ignore
            logger=logger,
        )

        protocol = urlparse(self.fsspec_config.remote_url).scheme  # type: ignore
        warnings.warn(
            f"`fsspec` protocol {protocol} is not directly supported by `unstructured`,"
            " so use it at your own risk. Supported protocols are `gcs`, `gs`, `s3`, `s3a`,"
            "`dropbox`, `abfs`, `az` and `sftp`.",
            UserWarning,
        )

    def get_source_connector_cls(self) -> t.Type[BaseSourceConnector]:
        from unstructured_ingest.connector.fsspec.fsspec import (
            FsspecSourceConnector,
        )

        return FsspecSourceConnector



import typing as t
from dataclasses import dataclass

from unstructured_ingest.interfaces import BaseDestinationConnector
from unstructured_ingest.runner.writers.base_writer import Writer

if t.TYPE_CHECKING:
    from unstructured_ingest.connector.mongodb import MongoDBWriteConfig, SimpleMongoDBConfig


@dataclass
class MongodbWriter(Writer):
    write_config: "MongoDBWriteConfig"
    connector_config: "SimpleMongoDBConfig"

    def get_connector_cls(self) -> t.Type[BaseDestinationConnector]:
        from unstructured_ingest.connector.mongodb import (
            MongoDBDestinationConnector,
        )

        return MongoDBDestinationConnector



import json
from dataclasses import dataclass
from datetime import date, datetime
from pathlib import Path
from typing import TYPE_CHECKING, Any, Optional

import numpy as np
import pandas as pd
from dateutil import parser
from pydantic import Field, Secret

from unstructured_ingest.utils.data_prep import batch_generator
from unstructured_ingest.utils.dep_check import requires_dependencies
from unstructured_ingest.utils.table import convert_to_pandas_dataframe
from unstructured_ingest.v2.interfaces import (
    AccessConfig,
    ConnectionConfig,
    FileData,
    Uploader,
    UploaderConfig,
    UploadStager,
    UploadStagerConfig,
)
from unstructured_ingest.v2.logger import logger
from unstructured_ingest.v2.processes.connector_registry import (
    DestinationRegistryEntry,
)

if TYPE_CHECKING:
    from singlestoredb.connection import Connection

CONNECTOR_TYPE = "singlestore"


class SingleStoreAccessConfig(AccessConfig):
    password: Optional[str] = Field(default=None, description="SingleStore password")


class SingleStoreConnectionConfig(ConnectionConfig):
    host: Optional[str] = Field(default=None, description="SingleStore host")
    port: Optional[int] = Field(default=None, description="SingleStore port")
    user: Optional[str] = Field(default=None, description="SingleStore user")
    database: Optional[str] = Field(default=None, description="SingleStore database")
    access_config: Secret[SingleStoreAccessConfig]

    @requires_dependencies(["singlestoredb"], extras="singlestore")
    def get_connection(self) -> "Connection":
        import singlestoredb as s2

        conn = s2.connect(
            host=self.host,
            port=self.port,
            database=self.database,
            user=self.user,
            password=self.access_config.get_secret_value().password,
        )
        return conn


class SingleStoreUploadStagerConfig(UploadStagerConfig):
    drop_empty_cols: bool = Field(default=False, description="Drop any columns that have no data")


@dataclass
class SingleStoreUploadStager(UploadStager):
    upload_stager_config: SingleStoreUploadStagerConfig

    @staticmethod
    def parse_date_string(date_string: str) -> date:
        try:
            timestamp = float(date_string)
            return datetime.fromtimestamp(timestamp)
        except Exception as e:
            logger.debug(f"date {date_string} string not a timestamp: {e}")
        return parser.parse(date_string)

    def run(
        self,
        elements_filepath: Path,
        file_data: FileData,
        output_dir: Path,
        output_filename: str,
        **kwargs: Any,
    ) -> Path:
        with open(elements_filepath) as elements_file:
            elements_contents = json.load(elements_file)
        output_path = Path(output_dir) / Path(f"{output_filename}.csv")
        output_path.parent.mkdir(parents=True, exist_ok=True)

        df = convert_to_pandas_dataframe(
            elements_dict=elements_contents,
            drop_empty_cols=self.upload_stager_config.drop_empty_cols,
        )
        datetime_columns = [
            "data_source_date_created",
            "data_source_date_modified",
            "data_source_date_processed",
        ]
        for column in filter(lambda x: x in df.columns, datetime_columns):
            df[column] = df[column].apply(self.parse_date_string)
        if "data_source_record_locator" in df.columns:
            df["data_source_record_locator"] = df["data_source_record_locator"].apply(
                lambda x: json.dumps(x) if x else None
            )

        with output_path.open("w") as output_file:
            df.to_csv(output_file, index=False)
        return output_path


class SingleStoreUploaderConfig(UploaderConfig):
    table_name: str = Field(description="SingleStore table to write contents to")
    batch_size: int = Field(default=100, description="Batch size when writing to SingleStore")


@dataclass
class SingleStoreUploader(Uploader):
    connection_config: SingleStoreConnectionConfig
    upload_config: SingleStoreUploaderConfig
    connector_type: str = CONNECTOR_TYPE

    def upload_csv(self, csv_path: Path) -> None:
        df = pd.read_csv(csv_path)
        logger.debug(
            f"uploading {len(df)} entries to {self.connection_config.database} "
            f"db in table {self.upload_config.table_name}"
        )
        stmt = "INSERT INTO {} ({}) VALUES ({})".format(
            self.upload_config.table_name,
            ", ".join(df.columns),
            ", ".join(["%s"] * len(df.columns)),
        )
        logger.debug(f"sql statement: {stmt}")
        df.replace({np.nan: None}, inplace=True)
        data_as_tuples = list(df.itertuples(index=False, name=None))
        with self.connection_config.get_connection() as conn:
            with conn.cursor() as cur:
                for chunk in batch_generator(
                    data_as_tuples, batch_size=self.upload_config.batch_size
                ):
                    cur.executemany(stmt, chunk)
                    conn.commit()

    def run(self, path: Path, file_data: FileData, **kwargs: Any) -> None:
        if path.suffix != ".csv":
            raise ValueError(f"Only .csv files are supported: {path}")
        self.upload_csv(csv_path=path)


singlestore_destination_entry = DestinationRegistryEntry(
    connection_config=SingleStoreConnectionConfig,
    uploader=SingleStoreUploader,
    uploader_config=SingleStoreUploaderConfig,
    upload_stager=SingleStoreUploadStager,
    upload_stager_config=SingleStoreUploadStagerConfig,
)



import hashlib
import json
import sys
import time
from dataclasses import dataclass, field
from datetime import timedelta
from pathlib import Path
from typing import TYPE_CHECKING, Any, Generator, List

from pydantic import Field, Secret

from unstructured_ingest.error import (
    DestinationConnectionError,
    SourceConnectionError,
    SourceConnectionNetworkError,
)
from unstructured_ingest.utils.data_prep import batch_generator, flatten_dict
from unstructured_ingest.utils.dep_check import requires_dependencies
from unstructured_ingest.v2.interfaces import (
    AccessConfig,
    ConnectionConfig,
    Downloader,
    DownloaderConfig,
    DownloadResponse,
    FileData,
    FileDataSourceMetadata,
    Indexer,
    IndexerConfig,
    Uploader,
    UploaderConfig,
    UploadStager,
    UploadStagerConfig,
    download_responses,
)
from unstructured_ingest.v2.logger import logger
from unstructured_ingest.v2.processes.connector_registry import (
    DestinationRegistryEntry,
    SourceRegistryEntry,
)

if TYPE_CHECKING:
    from couchbase.cluster import Cluster

CONNECTOR_TYPE = "couchbase"
SERVER_API_VERSION = "1"


class CouchbaseAccessConfig(AccessConfig):
    password: str = Field(description="The password for the Couchbase server")


class CouchbaseConnectionConfig(ConnectionConfig):
    username: str = Field(description="The username for the Couchbase server")
    bucket: str = Field(description="The bucket to connect to on the Couchbase server")
    connection_string: str = Field(
        default="couchbase://localhost", description="The connection string of the Couchbase server"
    )
    scope: str = Field(
        default="_default", description="The scope to connect to on the Couchbase server"
    )
    collection: str = Field(
        default="_default", description="The collection to connect to on the Couchbase server"
    )
    connector_type: str = Field(default=CONNECTOR_TYPE, init=False)
    access_config: Secret[CouchbaseAccessConfig]

    @requires_dependencies(["couchbase"], extras="couchbase")
    def connect_to_couchbase(self) -> "Cluster":
        from couchbase.auth import PasswordAuthenticator
        from couchbase.cluster import Cluster
        from couchbase.options import ClusterOptions

        auth = PasswordAuthenticator(self.username, self.access_config.get_secret_value().password)
        options = ClusterOptions(auth)
        options.apply_profile("wan_development")
        cluster = Cluster(self.connection_string, options)
        cluster.wait_until_ready(timedelta(seconds=5))
        return cluster


class CouchbaseUploadStagerConfig(UploadStagerConfig):
    pass


@dataclass
class CouchbaseUploadStager(UploadStager):
    upload_stager_config: CouchbaseUploadStagerConfig = field(
        default_factory=lambda: CouchbaseUploadStagerConfig()
    )

    def run(
        self,
        elements_filepath: Path,
        output_dir: Path,
        output_filename: str,
        **kwargs: Any,
    ) -> Path:
        with open(elements_filepath) as elements_file:
            elements_contents = json.load(elements_file)

        output_elements = []
        for element in elements_contents:
            new_doc = {
                element["element_id"]: {
                    "embedding": element.get("embeddings", None),
                    "text": element.get("text", None),
                    "metadata": element.get("metadata", None),
                    "type": element.get("type", None),
                }
            }
            output_elements.append(new_doc)

        output_path = Path(output_dir) / Path(f"{output_filename}.json")
        with open(output_path, "w") as output_file:
            json.dump(output_elements, output_file)
        return output_path


class CouchbaseUploaderConfig(UploaderConfig):
    batch_size: int = Field(default=50, description="Number of documents to upload per batch")


@dataclass
class CouchbaseUploader(Uploader):
    connection_config: CouchbaseConnectionConfig
    upload_config: CouchbaseUploaderConfig
    connector_type: str = CONNECTOR_TYPE

    def precheck(self) -> None:
        try:
            self.connection_config.connect_to_couchbase()
        except Exception as e:
            logger.error(f"Failed to validate connection {e}", exc_info=True)
            raise DestinationConnectionError(f"failed to validate connection: {e}")

    def run(self, path: Path, file_data: FileData, **kwargs: Any) -> None:
        with path.open("r") as file:
            elements_dict = json.load(file)
        logger.info(
            f"writing {len(elements_dict)} objects to destination "
            f"bucket, {self.connection_config.bucket} "
            f"at {self.connection_config.connection_string}",
        )
        cluster = self.connection_config.connect_to_couchbase()
        bucket = cluster.bucket(self.connection_config.bucket)
        scope = bucket.scope(self.connection_config.scope)
        collection = scope.collection(self.connection_config.collection)

        for chunk in batch_generator(elements_dict, self.upload_config.batch_size):
            collection.upsert_multi({doc_id: doc for doc in chunk for doc_id, doc in doc.items()})


class CouchbaseIndexerConfig(IndexerConfig):
    batch_size: int = Field(default=50, description="Number of documents to index per batch")


@dataclass
class CouchbaseIndexer(Indexer):
    connection_config: CouchbaseConnectionConfig
    index_config: CouchbaseIndexerConfig
    connector_type: str = CONNECTOR_TYPE

    def precheck(self) -> None:
        try:
            self.connection_config.connect_to_couchbase()
        except Exception as e:
            logger.error(f"Failed to validate connection {e}", exc_info=True)
            raise DestinationConnectionError(f"failed to validate connection: {e}")

    @requires_dependencies(["couchbase"], extras="couchbase")
    def _get_doc_ids(self) -> List[str]:
        query = (
            f"SELECT META(d).id "
            f"FROM `{self.connection_config.bucket}`."
            f"`{self.connection_config.scope}`."
            f"`{self.connection_config.collection}` as d"
        )

        max_attempts = 5
        attempts = 0
        while attempts < max_attempts:
            try:
                cluster = self.connection_config.connect_to_couchbase()
                result = cluster.query(query)
                document_ids = [row["id"] for row in result]
                return document_ids
            except Exception as e:
                attempts += 1
                time.sleep(3)
                if attempts == max_attempts:
                    raise SourceConnectionError(f"failed to get document ids: {e}")

    def run(self, **kwargs: Any) -> Generator[FileData, None, None]:
        ids = self._get_doc_ids()

        id_batches = [
            ids[i * self.index_config.batch_size : (i + 1) * self.index_config.batch_size]
            for i in range(
                (len(ids) + self.index_config.batch_size - 1) // self.index_config.batch_size
            )
        ]
        for batch in id_batches:
            # Make sure the hash is always a positive number to create identified
            identified = str(hash(tuple(batch)) + sys.maxsize + 1)
            yield FileData(
                identifier=identified,
                connector_type=CONNECTOR_TYPE,
                metadata=FileDataSourceMetadata(
                    url=f"{self.connection_config.connection_string}/"
                    f"{self.connection_config.bucket}",
                    date_processed=str(time.time()),
                ),
                additional_metadata={
                    "ids": list(batch),
                    "bucket": self.connection_config.bucket,
                },
            )


class CouchbaseDownloaderConfig(DownloaderConfig):
    fields: list[str] = field(default_factory=list)


@dataclass
class CouchbaseDownloader(Downloader):
    connection_config: CouchbaseConnectionConfig
    download_config: CouchbaseDownloaderConfig
    connector_type: str = CONNECTOR_TYPE

    def is_async(self) -> bool:
        return False

    def get_identifier(self, bucket: str, record_id: str) -> str:
        f = f"{bucket}-{record_id}"
        if self.download_config.fields:
            f = "{}-{}".format(
                f,
                hashlib.sha256(",".join(self.download_config.fields).encode()).hexdigest()[:8],
            )
        return f

    def map_cb_results(self, cb_results: dict) -> str:
        doc_body = cb_results
        flattened_dict = flatten_dict(dictionary=doc_body)
        str_values = [str(value) for value in flattened_dict.values()]
        concatenated_values = "\n".join(str_values)
        return concatenated_values

    def generate_download_response(
        self, result: dict, bucket: str, file_data: FileData
    ) -> DownloadResponse:
        record_id = result["id"]
        filename_id = self.get_identifier(bucket=bucket, record_id=record_id)
        filename = f"{filename_id}.txt"
        download_path = self.download_dir / Path(filename)
        logger.debug(
            f"Downloading results from bucket {bucket} and id {record_id} to {download_path}"
        )
        download_path.parent.mkdir(parents=True, exist_ok=True)
        try:
            with open(download_path, "w", encoding="utf8") as f:
                f.write(self.map_cb_results(cb_results=result))
        except Exception as e:
            logger.error(
                f"failed to download from bucket {bucket} "
                f"and id {record_id} to {download_path}: {e}",
                exc_info=True,
            )
            raise SourceConnectionNetworkError(f"failed to download file {file_data.identifier}")
        return DownloadResponse(
            file_data=FileData(
                identifier=filename_id,
                connector_type=CONNECTOR_TYPE,
                metadata=FileDataSourceMetadata(
                    version=None,
                    date_processed=str(time.time()),
                    record_locator={
                        "connection_string": self.connection_config.connection_string,
                        "bucket": bucket,
                        "scope": self.connection_config.scope,
                        "collection": self.connection_config.collection,
                        "document_id": record_id,
                    },
                ),
            ),
            path=download_path,
        )

    def run(self, file_data: FileData, **kwargs: Any) -> download_responses:
        bucket_name: str = file_data.additional_metadata["bucket"]
        ids: list[str] = file_data.additional_metadata["ids"]

        cluster = self.connection_config.connect_to_couchbase()
        bucket = cluster.bucket(bucket_name)
        scope = bucket.scope(self.connection_config.scope)
        collection = scope.collection(self.connection_config.collection)

        download_resp = self.process_all_doc_ids(ids, collection, bucket_name, file_data)
        return list(download_resp)

    def process_doc_id(self, doc_id, collection, bucket_name, file_data):
        result = collection.get(doc_id)
        return self.generate_download_response(
            result=result.content_as[dict], bucket=bucket_name, file_data=file_data
        )

    def process_all_doc_ids(self, ids, collection, bucket_name, file_data):
        for doc_id in ids:
            yield self.process_doc_id(doc_id, collection, bucket_name, file_data)

    async def run_async(self, file_data: FileData, **kwargs: Any) -> download_responses:
        raise NotImplementedError()


couchbase_destination_entry = DestinationRegistryEntry(
    connection_config=CouchbaseConnectionConfig,
    uploader=CouchbaseUploader,
    uploader_config=CouchbaseUploaderConfig,
    upload_stager=CouchbaseUploadStager,
    upload_stager_config=CouchbaseUploadStagerConfig,
)

couchbase_source_entry = SourceRegistryEntry(
    connection_config=CouchbaseConnectionConfig,
    indexer=CouchbaseIndexer,
    indexer_config=CouchbaseIndexerConfig,
    downloader=CouchbaseDownloader,
    downloader_config=CouchbaseDownloaderConfig,
)



import logging
import typing as t
from abc import ABC, abstractmethod
from dataclasses import dataclass

from unstructured_ingest.enhanced_dataclass import EnhancedDataClassJsonMixin
from unstructured_ingest.interfaces import (
    BaseConnectorConfig,
    BaseDestinationConnector,
    BaseSourceConnector,
    ChunkingConfig,
    EmbeddingConfig,
    PartitionConfig,
    PermissionsConfig,
    ProcessorConfig,
    ReadConfig,
    RetryStrategyConfig,
)
from unstructured_ingest.logger import ingest_log_streaming_init
from unstructured_ingest.processor import process_documents
from unstructured_ingest.runner.writers.base_writer import Writer


@dataclass
class Runner(EnhancedDataClassJsonMixin, ABC):
    connector_config: BaseConnectorConfig
    processor_config: ProcessorConfig
    read_config: ReadConfig
    partition_config: PartitionConfig
    writer: t.Optional[Writer] = None
    writer_kwargs: t.Optional[dict] = None
    embedding_config: t.Optional[EmbeddingConfig] = None
    chunking_config: t.Optional[ChunkingConfig] = None
    permissions_config: t.Optional[PermissionsConfig] = None
    retry_strategy_config: t.Optional[RetryStrategyConfig] = None

    def run(self, *args, **kwargs):
        ingest_log_streaming_init(logging.DEBUG if self.processor_config.verbose else logging.INFO)
        self.update_read_config()
        source_connector = self.get_source_connector()
        self.process_documents(
            source_doc_connector=source_connector,
        )

    @abstractmethod
    def update_read_config(self):
        pass

    @abstractmethod
    def get_source_connector_cls(self) -> t.Type[BaseSourceConnector]:
        pass

    def get_source_connector(self) -> BaseSourceConnector:
        source_connector_cls = self.get_source_connector_cls()
        return source_connector_cls(
            processor_config=self.processor_config,
            connector_config=self.connector_config,
            read_config=self.read_config,
        )

    def get_dest_doc_connector(self) -> t.Optional[BaseDestinationConnector]:
        writer_kwargs = self.writer_kwargs if self.writer_kwargs else {}
        if self.writer:
            return self.writer.get_connector(**writer_kwargs)
        return None

    def get_permissions_config(self) -> t.Optional[PermissionsConfig]:
        if self.permissions_config is None:
            return None

        permissions_config_filled = bool(
            self.permissions_config.application_id
            and self.permissions_config.client_cred
            and self.permissions_config.tenant,
        )

        return self.permissions_config if permissions_config_filled else None

    def process_documents(self, source_doc_connector: BaseSourceConnector):
        process_documents(
            processor_config=self.processor_config,
            source_doc_connector=source_doc_connector,
            partition_config=self.partition_config,
            dest_doc_connector=self.get_dest_doc_connector(),
            embedder_config=self.embedding_config,
            chunking_config=self.chunking_config,
            permissions_config=self.get_permissions_config(),
            retry_strategy_config=self.retry_strategy_config,
        )



import typing as t

from unstructured_ingest.cli.base.src import BaseSrcCmd
from unstructured_ingest.cli.cmds import base_src_cmd_fns


def get_src_cmd_map() -> t.Dict[str, t.Callable[[], BaseSrcCmd]]:
    return {b().cmd_name_key: b for b in base_src_cmd_fns}


def get_src_cmd(cmd_name: str) -> t.Callable[[], BaseSrcCmd]:
    return get_src_cmd_map()[cmd_name]
